//=============================================================================
// Base emitter class
//=============================================================================
class ParticleEmitter expands Actor
	Abstract;

#EXEC TEXTURE IMPORT FILE="S_Emitter.bmp" NAME="S_Emitter" GROUP="Icons" MIPS=off FLAGS=2 TEXFLAGS=0

struct FRange
{
	var() float Min,Max;
};
struct VRange
{
	var() FRange X,Y,Z;
};
struct IRange
{
	var() int Min,Max;
};
Enum ERangeType
{
	RT_Box,
	RT_Sphere,
	RT_Cylinder
};
struct HugeVecRange
{
	var() VRange BoxRange;
	var() FRange SphereRange;
	var() bool bPosRelativeToRot,bEffectedByLocation;
	var() ERangeType RangeType;
};
struct ParticleSndInfo
{
	var() ESoundSlot SoundSlot;
	var() float SoundPitch,SoundRadius,SoundVolume;
	var() Sound SoundName;
};

Static final function float ConvertFRange( FRange F )
{
	if( F.Min==F.Max )
		Return F.Min;
	Return F.Min+(F.Max-F.Min)*FRand();
}
Static final function vector ConvertVRange( VRange F )
{
	local vector V;
	
	V.X = ConvertFRange(F.X);
	V.Y = ConvertFRange(F.Y);
	V.Z = ConvertFRange(F.Z);
	Return V;
}
Static final function int ConvertIRange( IRange F )
{
	if( F.Min==F.Max )
		Return F.Min;
	Return F.Min+Rand(F.Max-F.Min);
}
Static final function vector ConvertHVRange( HugeVecRange F, optional rotator R, optional vector L )
{
	local vector V;
	
	if( F.bEffectedByLocation && L==vect(0,0,0) )
		L = VRand();
	if( F.RangeType==RT_Box )
		V = ConvertVRange(F.BoxRange);
	else if( F.RangeType==RT_Sphere )
	{
		if( F.bEffectedByLocation )
			V = Normal(L)*ConvertFRange(F.SphereRange);
		else V = VRand()*ConvertFRange(F.SphereRange);
	}
	else
	{
		if( F.bEffectedByLocation )
		{
			V = L;
			V.Z = 0;
		}
		else
		{
			V.X = 0.5-FRand();
			V.Y = 0.5-FRand();
		}
		V = Normal(V)*ConvertFRange(F.SphereRange);
	}
	if( F.bPosRelativeToRot )
		V = (V >> R);
	Return V;
}

simulated function ParticleDied( EmitterParticle Other );

// Fast check if player is rendering some point in world.
simulated function bool PositionCanBeSeen( vector Start, vector End, vector ViewDir )
{
	if( (Normal(End-Start) Dot ViewDir)<0.3 )
		Return False;
	Return True;
}

defaultproperties
{
				bHidden=True
				bStasis=True
				bDirectional=True
				Texture=Texture'PHT.Icons.S_Emitter'
}
